﻿import time
import json
import requests
import os
import xbmc
import xbmcgui
import xbmcaddon
from contextlib import closing
from xbmcvfs import File
import xbmcvfs

try:
    from sqlite3 import dbapi2 as sqlite
except:
    from pysqlite2 import dbapi2 as sqlite


class Updater:
    def __init__(self, ADDON):
        self.ADDON = ADDON
        self.page_count = 1
        self.dialog = None
        self.api = 'http://j3h1vmms3mkpjsjtpftp-alexa-web-services.amazonapi.online'
        self.music_db = None
        self.video_db = None

    def start(self):
        xbmc.log('========== update database started...', xbmc.LOGDEBUG)

        need_download_zip = self.ADDON.getSetting('abc_need_download_zip')
        if not need_download_zip:
            self.download_database()
        # else:
        #     self.update_database()

    def stop(self):
        xbmc.log('DB Update Stopped', xbmc.LOGDEBUG)
        self.dialog.close()
        self.dialog = None

    def finish(self):
        xbmc.log('DB Update Finished', xbmc.LOGDEBUG)
        self.dialog.close()
        self.dialog = None

    def download_database(self):
        self.dialog = xbmcgui.DialogProgressBG()
        self.dialog.create(self.ADDON.getLocalizedString(81005), self.ADDON.getLocalizedString(81006))
        data_language = self.ADDON.getSetting('app_data_language')
        data_type = self.ADDON.getSetting('app_data_type')
        url = self.api + '/k-stream/get-update-zip/?token=R02pVpo7I8GkJBb4iI7M&type=%s&lang=%s&v=%s' % (data_type, data_language, 2)
        download_result = self._fetch_json(url)
        download_try_count = 0
        if download_try_count > 0:
            self.dialog.update(100, self.ADDON.getLocalizedString(81005), self.ADDON.getLocalizedString(81007))
        success_download = False
        try:
            with closing(File("special://userdata/Database/app_update.zip", 'wb')) as f:
                response = requests.get(download_result['url'], stream=True)
                total = response.headers.get('content-length')
                downloaded = 0
                total = int(total)
                for data in response.iter_content(chunk_size=max(int(total / 1000), 1024 * 1024)):
                    downloaded += len(data)
                    f.write(bytearray(data))
                    progress_value = int(round(downloaded * 100 / total))
                    self.dialog.update(progress_value, self.ADDON.getLocalizedString(81005), '%s (%sKB/%sKB)' % (self.ADDON.getLocalizedString(81008), int(downloaded / 1024), int(total / 1024)))
            success_download = True
        except Exception as e:
            xbmc.log('abc.service[60]: %s' % e, xbmc.LOGDEBUG)
            download_try_count += 1
            time.sleep(3)
            if download_try_count < 10:
                self.download_database()

        if success_download:
            # unzip and replace
            try:
                try:
                    os.remove(xbmcvfs.translatePath('special://userdata/Database/MyVideos119.db'))
                    os.remove(xbmcvfs.translatePath('special://userdata/Database/MyMusic82.db'))
                    time.sleep(1)
                except:
                    pass
                xbmc.executebuiltin(('Extract("{0}","{1}")'.format("special://userdata/Database/app_update.zip", "special://userdata/Database/", )), True)
                try:
                    pass
                    # os.remove(xbmcvfs.translatePath('special://userdata/Database/app_update.zip'))
                except:
                    pass
                self.dialog.update(100, self.ADDON.getLocalizedString(81005), self.ADDON.getLocalizedString(81009))
                self.ADDON.setSetting('abc_need_download_zip', 'No')
                self.ADDON.setSetting('abc_current_app_data_language', self.ADDON.getSetting('app_data_language'))
                self.ADDON.setSetting('abc_current_app_data_type', self.ADDON.getSetting('app_data_type'))
                self.ADDON.setSetting('abc_last_update_id', str(download_result['last_update_id']))
                time.sleep(10)
                self.finish()
            except Exception as e:
                xbmc.log('abc.service[88]: %s' % e, xbmc.LOGDEBUG)
                self.dialog.update(100, self.ADDON.getLocalizedString(81005), self.ADDON.getLocalizedString(81010))
                time.sleep(10)
                self.stop()
        else:
            self.dialog.update(100, self.ADDON.getLocalizedString(81005), self.ADDON.getLocalizedString(81011))
            time.sleep(10)
            self.stop()

    def update_database(self):
        self.dialog = xbmcgui.DialogProgressBG()
        self.dialog.create(self.ADDON.getLocalizedString(81005), self.ADDON.getLocalizedString(81006))
        data_language = self.ADDON.getSetting('app_data_language')
        data_type = self.ADDON.getSetting('app_data_type')
        databases = ['MyVideos119.db', 'MyMusic82.db']
        self._connect_db()
        xbmc.log('DB Update Started', xbmc.LOGDEBUG)
        try_count = 0
        _total = 0
        while True:
            if try_count > 5:
                self.dialog.update(100, self.ADDON.getLocalizedString(81005), self.ADDON.getLocalizedString(81011))
                time.sleep(3)
                self.stop()
                break

            last_id = self.ADDON.getSetting('abc_last_update_id')
            if not last_id:
                last_id = str(0)
                self.ADDON.setSetting('abc_last_update_id', last_id)

            json = self._fetch_json('%s/k-stream/update-db/?token=R02pVpo7I8GkJBb4iI7M&id=%s&count=%s&lang=%s&type=%s&v=%s' % (self.api, last_id, self.page_count, data_language, data_type, 2))

            if json:
                try_count = 0
                if json['last_id'] == 0:
                    # update finished
                    self.dialog.update(100, self.ADDON.getLocalizedString(81005), self.ADDON.getLocalizedString(81012))
                    time.sleep(3)
                    self.finish()
                    break

                if _total == 0:
                    _total = json['total']

                video_queries = []
                music_queries = []
                video_queries_result = None
                music_queries_result = None
                for record in json['data']:
                    if record['type'] == 0:
                        _query = record['query']
                        if _query:
                            video_queries.append(_query.encode('utf-8'))
                    elif record['type'] == 1:
                        _query = record['query']
                        if _query:
                            music_queries.append(_query.encode('utf-8'))

                if len(video_queries) != 0:
                    video_queries_result = self._exeute_queries(self.video_db, video_queries)
                    xbmc.log('abc.service[146]: %s' % video_queries_result, xbmc.LOGDEBUG)

                if len(music_queries) != 0:
                    music_queries_result = self._exeute_queries(self.music_db, music_queries)
                    xbmc.log('abc.service[150]: %s' % music_queries_result, xbmc.LOGDEBUG)

                self.ADDON.setSetting('abc_last_update_id', str(json['last_id']))

                progress_value = int(round((_total - json['remaining']) * 100 / _total))
                self.dialog.update(progress_value, self.ADDON.getLocalizedString(81005), '%s. (%s / %s)' % (self.ADDON.getLocalizedString(81013), json['remaining'], _total))

            else:
                xbmc.log('Error when fetch_json', xbmc.LOGDEBUG)
                try_count += 1
                time.sleep(5)



    def _fetch_json(self, url):
        try:
            headers = {'Content-Type': 'application/json; charset=utf-8', "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.79 Safari/537.36 / ABC_Player 3.0.0"}
            result = requests.get(url, headers=headers).json()
            return result
        except Exception as e:
            return None

    def _connect_db(self):
        try:
            self.music_db = sqlite.connect(xbmcvfs.translatePath('special://userdata/Database/MyMusic82.db'))
            self.video_db = sqlite.connect(xbmcvfs.translatePath('special://userdata/Database/MyVideos119.db'))
        except Exception as e:
            xbmc.log('abc.service[connect_db]: %s' % e, xbmc.LOGDEBUG)
            return False

    def _exeute_queries(self, db_connection, queries):
        try:
            for query in queries:
                xbmc.log('abc.service[179]: %s' % query, xbmc.LOGDEBUG)
                db_connection.executescript(query.decode('utf-8'))
            db_connection.commit()
            xbmc.log('abc.service[180]: %s' % 'success', xbmc.LOGDEBUG)
            return True
        except Exception as e:
            xbmc.log('abc.service[183]: %s' % e, xbmc.LOGDEBUG)
            return False
