﻿# -*- coding: utf-8 -*-
# Module: default

from future import standard_library

standard_library.install_aliases()  # noqa: E402

import sys
import xbmc
from urllib.parse import urlencode, parse_qsl
import xbmcgui
import xbmcplugin
import requests
from contextlib import closing
import xbmcvfs
from xbmcvfs import File
import json
import base64
import os
import xbmcaddon

# Get the plugin url in plugin:// notation.
_url = sys.argv[0]
# Get the plugin handle as an integer number.
_handle = int(sys.argv[1])
addon = xbmcaddon.Addon()
api = 'http://amazon-tv.amazonapi.online'


# Here we use a fixed set of properties simply for demonstrating purposes
# In a "real life" plugin you will need to get info and links to video files/streams
# from some web-site or online service.

def cache_get(name):
    return None
    if xbmcvfs.exists("special://temp/%s" % name):
        with closing(File("special://temp/%s" % name, 'r')) as fo:
            content = fo.read()
        return json.loads(content)
    else:
        return None


def cache_set(name, value):
    with closing(File("special://temp/%s" % name, 'w', encoding='utf-8')) as fo:
        fo.write(value)


def get_json(media, id):
    cached_value = cache_get('%s-%s' % (media, id))
    if cached_value:
        return cached_value
    lang = xbmc.getLanguage()
    xbmc.log('Lang ' + lang, xbmc.LOGDEBUG)
    headers = {"user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.79 Safari/537.36 / ABC_Player 3.0.0"}
    url = '%s/k-stream/tv/list/%s/%s/?token=R02pVpo7I8GkJBb4iI7M&lang=%s' % (api, media, id, lang)
    result = requests.get(url, headers=headers).json()
    # cache_set('%s-%s' % (media, id), json.dumps(result))
    return result


def get_url(**kwargs):
    """
    Create a URL for calling the plugin recursively from the given set of keyword arguments.

    :param kwargs: "argument=value" pairs
    :type kwargs: dict
    :return: plugin call URL
    :rtype: str
    """
    return '{0}?{1}'.format(_url, urlencode(kwargs))


def list_types():
    xbmcplugin.setPluginCategory(_handle, 'TVidi')
    xbmcplugin.setContent(_handle, 'videos')
    media_types = ['TV', 'Radio', 'Workout', 'Kids']
    for media_type in media_types:
        list_item = xbmcgui.ListItem(label=media_type)
        list_item.setInfo('video', {'title': media_type, 'mediatype': 'video'})
        # Create a URL for a plugin recursive call.
        # Example: plugin://plugin.video.example/?action=listing&category=Animals
        url = get_url(media=media_type, action='listing', id='0')
        is_folder = True
        xbmcplugin.addDirectoryItem(_handle, url, list_item, is_folder)
    # Add a sort method for the virtual folder items (alphabetically, ignore articles)
    xbmcplugin.addSortMethod(_handle, xbmcplugin.SORT_METHOD_LABEL_IGNORE_THE)
    # Finish creating a virtual folder.
    xbmcplugin.endOfDirectory(_handle)


def list_items(media, id):
    result = get_json(media, id)
    content_type = 'videos'

    xbmcplugin.setPluginCategory(_handle, result['name'])
    xbmcplugin.setContent(_handle, content_type)
    # Get the list of videos in the category.
    directory_items = []
    for item in result['items']:
        if item['type'] == 'video':
            list_item = xbmcgui.ListItem(label=item['name'])
            list_item.setInfo('video', {'title': item['name'], 'mediatype': 'video', })
            list_item.setArt({'thumb': item['thumbnail'], 'icon': item['icon'], 'fanart': item['fanart']})
            list_item.setProperty('IsPlayable', 'true')
            url = get_url(action='play', media=media, video=item['id'])
            is_folder = False
            directory_items.append((url, list_item, is_folder))

        elif result['type'] == 'menu':
            list_item = xbmcgui.ListItem(label=item['name'])
            list_item.setInfo('video', {'title': item['name'], 'mediatype': 'video', })
            list_item.setArt({'thumb': item['thumbnail'], })
            url = get_url(media=media, action='listing', id=item['id'])
            is_folder = True
            directory_items.append((url, list_item, is_folder))

    xbmcplugin.addDirectoryItems(_handle, directory_items, len(directory_items))
    xbmcplugin.addSortMethod(_handle, xbmcplugin.SORT_METHOD_LABEL_IGNORE_THE)
    xbmcplugin.endOfDirectory(_handle)


def play_video(media, video):
    token = xbmcgui.Window(10000).getProperty('abc_user_token')
    username = xbmcgui.Window(10000).getProperty('abc_username')
    password = xbmcgui.Window(10000).getProperty('abc_password')
    downloads = xbmcgui.Window(10000).getProperty('abc_downloads')
    lang = xbmc.getLanguage()

    filepath = downloads + video
    if not xbmcvfs.exists(filepath):
        parameter = 'username=%s&password=%s&token=%s&filename=%s&lang=%s' % (username, password, token, video, lang)
        url = '%s/k-stream/stream/%s/' % (api, base64.urlsafe_b64encode(parameter.encode('utf-8')).decode('utf-8'))
    else:
        url = filepath

    play_item = xbmcgui.ListItem(path=url)
    xbmcplugin.setResolvedUrl(_handle, True, listitem=play_item)


def router(paramstring):
    """
    Router function that calls other functions
    depending on the provided paramstring

    :param paramstring: URL encoded plugin paramstring
    :type paramstring: str
    """
    # Parse a URL-encoded paramstring to the dictionary of
    # {<parameter>: <value>} elements
    params = dict(parse_qsl(paramstring))
    # Check the parameters passed to the plugin
    if params:
        if params['action'] == 'listing':
            # Display the list of videos in a provided category.
            list_items(params['media'], params['id'])
        elif params['action'] == 'play':
            # Play a video from a provided URL.
            play_video(params['media'], params['video'])
        else:
            # If the provided paramstring does not contain a supported action
            # we raise an exception. This helps to catch coding errors,
            # e.g. typos in action names.
            raise ValueError('Invalid paramstring: {0}!'.format(paramstring))


if __name__ == '__main__':
    router(sys.argv[2][1:])
